#include "sdlab.h"

const char *prefix = "<!DOCTYPE>\r\n<html lang=\"ja\">\r\n  <head>\r\n    <meta http-equiv=\"content-type\" content=\"text/html; charset=UTF-8\" />\r\n    <meta http-equiv=\"content-style-type\" content=\"text/css\" />\r\n    <meta http-equiv=\"content-script-type\" content=\"text/javascript\" />\r\n    <meta name=\"author\" content=\"Shunsuke Saruwatari\" />\r\n    <meta name=\"copyright\" content=\"(C) 2013 Shunsuke Saruwatari\" />\r\n\r\n    <title>SDLab: Constellation Graph</title>\r\n<style type=\"text/css\">\r\n<!--\r\n* {\r\n  margin: 0;\r\n  padding: 0;\r\n}\r\n\r\nbody {\r\n  width: 100%%;\r\n}\r\n\r\nh1 {\r\n border-left : 5px solid #0000CC;\r\n margin : 20px 20px 20px 5px;\r\n padding-left : 10px;\r\n}\r\n\r\n#body {\r\n  position: relative;\r\n  margin: 0 auto;\r\n  width: 100%%; \r\n  background-color: #ffffff;\r\n}\r\n\r\n\r\n\r\n/*************/\r\n/*** #main ***/\r\n/*************/\r\n\r\n#main {\r\n  float: left;\r\n  width: 800px;\r\n  line-height: 130%%;\r\n  padding: 0px 0px 0px 10px;\r\n}\r\n\r\na:link, a:visited {\r\n text-decoration: none;\r\n color: #0000FF;\r\n}\r\n\r\na:hover {\r\n text-decoration: underline;\r\n color: #000000;\r\n}\r\n\r\ncanvas {\r\n border-left : 1px solid #0000FF;\r\n border-bottom : 1px solid #0000FF;\r\n border-right : 1px solid #0000FF;\r\n border-top : 1px solid #0000FF;\r\n margin : 5px 5px 5px 20px;\r\n width : 480px;\r\n height: 480px;\r\n}\r\n-->\r\n</style>\r\n<script type=\"text/javascript\">\r\n\r\nvar ctx_iq;\r\n\r\nonload = function() {\r\n  canvas_iq = document.getElementById('canvas_iq');\r\n  ctx_iq = canvas_iq.getContext('2d');\r\n\r\n/*********** from here *************/\r\n  var str_iq = \"[";

  const char* suffix = "[0,0]]\";\r\n/*********** till here *************/\r\n\r\n  data_iq = JSON.parse(str_iq);\r\n\r\n  plotDot(ctx_iq, data_iq, 'rgb(255, 0, 0)');\r\n};\r\n\r\nfunction plotDot(ctx, data, color)\r\n{\r\n  var x, y, len;\r\n  ctx.strokeStyle = color;\r\n  ctx.clearRect(0, 0, 480, 480);\r\n  y = data[0][0] * 120 + 240;\r\n  x = data[1][1] * 120 + 240;\r\n  len = data.length;\r\n\r\n  ctx.beginPath();\r\n  ctx.arc(x, y, 0.5, 0, Math.PI * 2, false);\r\n  ctx.stroke();\r\n\r\n  step = 1000 / (len - 1);\r\n\r\n  for(i = 0; i < len - 1; i++){\r\n    x = data[i][0] * 120 + 240;\r\n    y = data[i][1] * 120 + 240;\r\n    ctx.beginPath();\r\n    ctx.arc(x, y, 0.5, 0, Math.PI * 2, false);\r\n    ctx.stroke();\r\n  }\r\n}\r\n\r\n</script>\r\n</head>\r\n<body>\r\n  <h1><a href=\"http://www.sdlabo.org/\">sdlabo.org</a>: Constellation Graph</h1>\r\n  <div id=\"main\">\r\n    <canvas id=\"canvas_iq\" width=\"480px\" height=\"480px\"></canvas>\r\n  </div>\r\n</body>\r\n</html>\r\n";

FILE* gfp;

void open_graph_constellation()
{
  gfp = fopen("constellatoin.html", "w");
  fprintf(gfp, prefix);
}

void graph_constellation(sdlab_complex s)
{
  fprintf(gfp, "[%f,%f],", s.real(), s.imag());
}

void close_graph_constellation()
{
  fprintf(gfp, suffix);
  fclose(gfp);
}

int alloc_count = 0;

sdlab_complex** cmat_alloc(int n, int m)
{
  sdlab_complex **ret = (sdlab_complex**)malloc(n * sizeof(sdlab_complex*));
  alloc_count++;
  for(int i = 0; i < n; i++){
    ret[i] = (sdlab_complex*)malloc(m * sizeof(sdlab_complex));
    alloc_count++;
  }

  return ret;
}

void cmat_free(sdlab_complex** p, int n, int m)
{
  for(int i = 0; i < n; i++){
    free(p[i]);
    alloc_count--;
  }

  free(p);
  alloc_count--;
}

void make_qam(int size, sdlab_complex **qam)
{
  int n = sqrt(size);
  int qam_bits = round(log2(n));
  uint16_t qam_mask = ((1 << qam_bits) - 1);
  double scale = sqrt((2 * (pow(2, qam_bits * 2) - 1)) / 3);

  for(uint16_t i = 0; i < n; i++){
    for(uint16_t j = 0; j < n; j++){
      double real = j;
      double imag = n - i - 1;

      real = real * 2 - qam_mask;
      imag = imag * 2 - qam_mask;
      real = real / scale;
      imag = imag / scale;

      qam[i][j] = sdlab_complex(real,imag);
    }
  }
}

int main(int argc, char** argv)
{
  if(argc != 2){
    printf("usage: %s [4096|1024|256|64|16|4]\n", argv[0]);
    exit(-1);
  }

  int size = atoi(argv[1]);
  int n = round(sqrt(size));

  sdlab_complex **qam = cmat_alloc(n, n);
  make_qam(size, qam);

  open_graph_constellation();
  for(int i = 0; i < n; i++){
    for(int j = 0; j < n; j++){
      printf("%f,%f\n", qam[i][j].real(), qam[i][j].imag());
      graph_constellation(qam[i][j]);
    }
  }
  close_graph_constellation();

  cmat_free(qam, n, n);
}
